/******************************************************************************
    Program url2data, builds database from url address:
    - main.c - main source file for print usage and start creating database.

    Copyright (C) 1997-2000 Silvio Klaic, sklaic@student.math.hr
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
******************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "url2data.h"

short int loadconf(char *FNAME);
void update(void);
void dumpfromfile(void);
short int geturlloc(char *UrlLoc);
void extrfname(char *Path, char *Fname);/* extract file name */
void listdata(void);
void go_recursive(void);

void usage(char *fname) /* write usage */
{
printf("Usage: %s [options] [url_address_for_retrieve]\n",fname);
printf("Options: -d <database>\tSet name for database.\n");
printf("\t\tBy default database file is created in current directory!\n");
printf("\t -e\tIf database contain requested url location, don't retrieve.\n");
printf("\t -f <url_file>\tRetrieve url location from list in \"url_file\".\n");
printf("\t -h\tWrite this help.\n");
printf("\t -l\tList database, report number of words and letters in it.\n");
printf("\t -L\tList database, report url location, number of words and\n");
printf("\t\tletters in it.\n");
printf("\t -r\tGo recursive from url location, follow only url location if\n");
printf("\t\tthey are subdirectories of url path with selected extensions\n");
printf("\t\tin configuration file.\n");
printf("\t -u\tUpdate database, check url from database and update only if\n");
printf("\t\ton url location is new data.\n");
printf("\t -U\tFull update database, update all urls from database.\n");
printf("Version 0.0.2, see man page for more information.\n");
exit(0);
}

int main(int argc, char *argv[]) /************ main program *************/
{
FILE *f;
int i;

if(argc<2) usage(argv[0]);/* no entry in arguments type usage */
/* reset options variable */
D_OPT=false;E_OPT=false;F_OPT=false;H_OPT=false;R_OPT=false;U_OPT=false;
database[0]=0;url_file_adr[0]=0;L_OPT=false;
for(i=1;i<argc;i++){/* go thru arguments to find options */
 if( (argv[i][0]=='-')&&(strlen(argv[i])==2) ){/* found options */
  switch (argv[i][1]) {/* test which option */
   case 'd': /* D option is selected */
	   D_OPT=true;i++;
	   if(i<argc) strcpy(database,argv[i]);
	   else database[0]=0;break;
   case 'e': /* E option is selected */
	   E_OPT=true;break;
   case 'f': /* F option is selected */
	   F_OPT=true;i++;
	   if(i<argc) strcpy(url_file_adr,argv[i]);
	   else url_file_adr[0]=0;break;
   case 'h': /* H option is selected */
	   H_OPT=true;break;
   case 'l': /* L option is selected */
	   L_OPT=true;break;
   case 'L': /* L option with url is selected */
	   L_OPT=2;break;
   case 'r': /* R option is selected */
	   R_OPT=true;break;
   case 'u': /* U option is selected */
	   U_OPT=true;break;
   case 'U': /* U full option is selected */
	   U_OPT=2;break;
   default : /* unknown option write error */
	   printf("\nUnknown option %s !\n\n",argv[i]);
	   usage(argv[0]);break;
  }/* end switch */
 }else{/* not a option, then it's url location */
  strcpy(url_file_adr,argv[i]);
 }
}/* end for i thru options */

if(H_OPT) usage(argv[0]);/* help needed */
/* test each option for all needed data */
if( (!L_OPT)&&(!U_OPT)&&(url_file_adr[0]==0) ){/* no url or url file list */
 if(F_OPT){/* no url file list */
  printf("\nThere is no url file list location for -f option !\n\n");
  usage(argv[0]);
 }else{/* no url location */
  printf("\nThere's no url location for download!\n\n");
  usage(argv[0]);
 }/* end else if no url */
}/* end if url location */
if(database[0]==0){/* no database location using default or fail */
 if(!D_OPT){/* no need to find default location for database */
  getcwd(database,1024);if(!U_OPT&&F_OPT){/* set file name as default */
   extrfname(url_file_adr,recdbase);/* get file name */
   strcat(database,"/");strcat(database,recdbase);strcat(database,".sdat");
  }else strcat(database,"/default.sdat");/* standard name */
  printf("Using default database: %s\n",database);
 }else{ /* no database file was specified but option is used */
  printf("\nThere is no database location for -d option !\n\n");
  usage(argv[0]);
 }/* end if no option */
 f=fopen(database,"ab");fclose(f);/* create if don't exists */
 if(access(database,W_OK & R_OK)!=0){/* fail to write database file */
  printf("Error \"%s\": %s !\n",database,strerror(errno));/* write error */
  exit(1);}/* can't continue */
}/* end if database named */

/* Updating values from config file if exists in conf file */
if (loadconf(CONF_FILE)) exit(1);
/**************** create all needed temp locations ************************/
if( (tmploc[0]!=0)&&(tmploc[strlen(tmploc)-1]!='/') ) strcat(tmploc,"/");
if(R_OPT||U_OPT){/* if there is recursive download make temp location */
 if(tmploc[0]!=0){/* temp location exists */
  sprintf(recdbase,"%s.u2d_recXXXXXX",tmploc);mktemp(recdbase);/* temp file */
 }else{/* no cfg temp file use default */
  strcpy(recdbase,"/tmp/.u2d_recXXXXXX");mktemp(recdbase);/* temp file */
 }/* end else def conf temp */
 creat(recdbase,S_IRUSR|S_IWUSR);/* create or truncate temp file */
 if(access(recdbase,W_OK & R_OK)!=0){/* fail to write rec temp file */
  printf("Error \"%s\": %s !\n",recdbase,strerror(errno));/* write error */
  exit(1);}/* can't continue */
}else recdbase[0]=0;/* there no need temp location, no recursive option */
/* create temp location for download and processing */
if(tmploc[0]!=0){/* temp location exists */
 sprintf(fixhtml,"%s.u2d_htmXXXXXX",tmploc);mktemp(fixhtml);/* temp file */
 sprintf(logdbase,"%s.u2d_logXXXXXX",tmploc);mktemp(logdbase);/* temp file */
 strcat(tmploc,".u2d_tmpXXXXXX");mktemp(tmploc);/* temp file */
}else{/* no cfg temp file use default */
 strcpy(fixhtml,"/tmp/.u2d_htmXXXXXX");mktemp(fixhtml);/* temp file */
 strcpy(logdbase,"/tmp/.u2d_logXXXXXX");mktemp(logdbase);/* temp file */
 strcpy(tmploc,"/tmp/.u2d_tmpXXXXXX");mktemp(tmploc);/* temp file */
}/* end else def conf temp */
creat(fixhtml,S_IRUSR|S_IWUSR);/* create or truncate temp file */
creat(logdbase,S_IRUSR|S_IWUSR);creat(tmploc,S_IRUSR|S_IWUSR);
if(access(fixhtml,W_OK & R_OK)!=0){/* fail to write log file */
 printf("Error \"%s\": %s !\n",fixhtml,strerror(errno));/* write error */
 exit(1);}/* can't continue */
if(access(logdbase,W_OK & R_OK)!=0){/* fail to write log file */
 printf("Error \"%s\": %s !\n",logdbase,strerror(errno));/* write error */
 exit(1);}/* can't continue */
if(access(tmploc,W_OK & R_OK)!=0){/* fail to write tmp file */
 printf("Error \"%s\": %s !\n",tmploc,strerror(errno));/* write error */
 exit(1);}/* can't continue */

/* write task what it will be done */
if(L_OPT) printf("Analyzing database %s...\n",database);
else{/* do other */
 if( E_OPT&&!U_OPT ) printf("Avoiding urls who are in database.\n");
 if(R_OPT) printf("Doing recursive update from url location.\n");
 if(U_OPT==1) printf("Updating modify url only from database...\n");
 if(U_OPT==2) printf("Updating all urls from database...\n");
 if(F_OPT) printf("Retrieving url locations from list file...\n");
}/* end else L option */
 
/* check options and start program */
if(L_OPT) listdata();/* print database info */
else if(U_OPT) update();/* check database and test url locations */
else if(F_OPT) dumpfromfile();/* read url from file */
else geturlloc(url_file_adr);/* read url */

if(R_OPT) go_recursive();/* do recursive retrive */

/* remove temp files */
printf("\r%79s\rRemoving temp files..."," ");
if(fixhtml[0]!=0) remove(fixhtml);
if(recdbase[0]!=0) remove(recdbase);
if(logdbase[0]!=0) remove(logdbase);
if(tmploc[0]!=0) remove(tmploc);
printf(" Done.\n");
return(0);
}
